@@ -33,11 +33,10 @@
 
 #include <string.h>
 
+#include <glib/gi18n.h>
 #include <glade/glade-xml.h>
-#include <libgnome/libgnome.h>
+#include <libgnome/gnome-help.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/libgnomeui.h>
-
 #include <gtksourceview/gtksourcetag.h>
 #include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcelanguagesmanager.h>
@@ -103,6 +102,12 @@ struct _GeditPreferencesDialog
 	
 	/* Line numbers */
 	GtkWidget	*display_line_numbers_checkbutton;
+
+	/* Highlight current line */
+	GtkWidget	*highlight_current_line_checkbutton;
+	
+	/* Highlight matching bracket */
+	GtkWidget	*bracket_matching_checkbutton;
 	
 	/* Right margin */
 	GtkWidget	*right_margin_checkbutton;
@@ -143,6 +148,13 @@ dialog_destroyed (GtkObject  *obj,
 	}	
 }
 
+static void
+destroy_dialog (GeditPreferencesDialog *dialog)
+{
+	g_object_unref (dialog->tooltips);
+	gtk_widget_destroy (dialog->dialog);
+}
+
 static void
 dialog_response_handler (GtkDialog              *dlg, 
 			 gint                    res_id,
@@ -163,7 +175,7 @@ dialog_response_handler (GtkDialog              *dlg,
 			break;
 
 		default:
-			gtk_widget_destroy (dialog->dialog);
+			destroy_dialog (dialog);
 	}
 }
 
@@ -294,6 +306,7 @@ setup_editor_page (GeditPreferencesDialog *dlg)
 	gboolean auto_save;
 	gint undo_levels;
 	gboolean can_set;
+	gint auto_save_interval;
 
 	gedit_debug (DEBUG_PREFS, "");
 
@@ -313,8 +326,13 @@ setup_editor_page (GeditPreferencesDialog *dlg)
 	auto_save = gedit_prefs_manager_get_auto_save ();
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->auto_save_checkbutton),
 				      auto_save );
+
+	auto_save_interval = gedit_prefs_manager_get_auto_save_interval ();
+	if (auto_save_interval <= 0)
+		auto_save_interval = GPM_DEFAULT_AUTO_SAVE_INTERVAL;
+
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (dlg->auto_save_spinbutton),
-				   gedit_prefs_manager_get_auto_save_interval ());
+				   auto_save_interval);
 
 	undo_levels = gedit_prefs_manager_get_undo_actions_limit ();
 	
@@ -399,6 +417,27 @@ display_line_numbers_checkbutton_toggled (GtkToggleButton        *button,
 	gedit_prefs_manager_set_display_line_numbers (gtk_toggle_button_get_active (button));
 }
 
+static void
+highlight_current_line_checkbutton_toggled (GtkToggleButton        *button,
+					  GeditPreferencesDialog *dlg)
+{
+	g_return_if_fail (button == 
+			GTK_TOGGLE_BUTTON (dlg->highlight_current_line_checkbutton));
+
+	gedit_prefs_manager_set_highlight_current_line (gtk_toggle_button_get_active (button));
+}
+
+static void
+bracket_matching_checkbutton_toggled (GtkToggleButton        *button,
+				      GeditPreferencesDialog *dlg)
+{
+	g_return_if_fail (button == 
+			GTK_TOGGLE_BUTTON (dlg->bracket_matching_checkbutton));
+
+	gedit_prefs_manager_set_bracket_matching (
+				gtk_toggle_button_get_active (button));
+}
+
 static gboolean split_button_state = TRUE;
 
 static void
@@ -482,6 +521,12 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	/* Set initial state */
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->display_line_numbers_checkbutton),
 				gedit_prefs_manager_get_display_line_numbers ());
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->highlight_current_line_checkbutton),
+				gedit_prefs_manager_get_highlight_current_line ());
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->bracket_matching_checkbutton),
+				      gedit_prefs_manager_get_bracket_matching ());
 
 	wrap_mode = gedit_prefs_manager_get_wrap_mode ();
 	switch (wrap_mode )
@@ -522,6 +567,12 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	gtk_widget_set_sensitive (dlg->display_line_numbers_checkbutton,
 				  gedit_prefs_manager_display_line_numbers_can_set ());
 
+	gtk_widget_set_sensitive (dlg->highlight_current_line_checkbutton,
+				  gedit_prefs_manager_highlight_current_line_can_set ());
+
+	gtk_widget_set_sensitive (dlg->bracket_matching_checkbutton,
+				  gedit_prefs_manager_bracket_matching_can_set ());
+
 	wrap_mode_can_set = gedit_prefs_manager_wrap_mode_can_set ();
 
 	gtk_widget_set_sensitive (dlg->wrap_text_checkbutton, 
@@ -542,6 +593,12 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	g_signal_connect (G_OBJECT (dlg->display_line_numbers_checkbutton), "toggled", 
 			  G_CALLBACK (display_line_numbers_checkbutton_toggled), 
 			  dlg);
+	g_signal_connect (G_OBJECT (dlg->highlight_current_line_checkbutton), "toggled", 
+			  G_CALLBACK (highlight_current_line_checkbutton_toggled), 
+			  dlg);
+	g_signal_connect (G_OBJECT (dlg->bracket_matching_checkbutton), "toggled", 
+			  G_CALLBACK (bracket_matching_checkbutton_toggled), 
+			  dlg);
 	g_signal_connect (G_OBJECT (dlg->wrap_text_checkbutton), "toggled", 
 			  G_CALLBACK (wrap_mode_checkbutton_toggled), 
 			  dlg);
@@ -1237,6 +1294,10 @@ get_preferences_dialog (GtkWindow *parent)
 
 	dialog->dialog = glade_xml_get_widget (gui, "preferences_dialog");
 	dialog->display_line_numbers_checkbutton = glade_xml_get_widget (gui, "display_line_numbers_checkbutton");
+	dialog->highlight_current_line_checkbutton = glade_xml_get_widget (gui, "highlight_current_line_checkbutton");
+
+	dialog->bracket_matching_checkbutton = glade_xml_get_widget (gui, "bracket_matching_checkbutton");
+
 	dialog->wrap_text_checkbutton = glade_xml_get_widget (gui, "wrap_text_checkbutton");
 	dialog->split_checkbutton = glade_xml_get_widget (gui, "split_checkbutton");
 
@@ -1291,6 +1352,8 @@ get_preferences_dialog (GtkWindow *parent)
 
 	if (!dialog->dialog ||
 	    !dialog->display_line_numbers_checkbutton ||
+	    !dialog->highlight_current_line_checkbutton ||
+	    !dialog->bracket_matching_checkbutton ||
 	    !dialog->wrap_text_checkbutton ||
 	    !dialog->split_checkbutton ||
 	    !dialog->default_font_checkbutton ||
@@ -1345,9 +1408,11 @@ get_preferences_dialog (GtkWindow *parent)
 
 		return NULL;
 	}
-	
+
 	dialog->tooltips = gtk_tooltips_new ();
-		
+	g_object_ref (dialog->tooltips);
+	gtk_object_sink (GTK_OBJECT (dialog->tooltips));
+
 	setup_editor_page (dialog);
 	setup_view_page (dialog);
 	setup_font_colors_page (dialog);
@@ -1368,7 +1433,6 @@ get_preferences_dialog (GtkWindow *parent)
 	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
 
 	return dialog;
-
 }
 
 void
