@@ -33,10 +33,11 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n.h>
+
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <eel/eel-string.h>
 
 #include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcelanguagesmanager.h>
@@ -61,8 +62,6 @@
 #include "gedit-languages-manager.h"
 #include "dialogs/gedit-close-confirmation-dialog.h"
 
-#include <eel/eel-alert-dialog.h>
-
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-control.h>
 
@@ -96,7 +95,17 @@ static void gedit_mdi_app_created_handler	(BonoboMDI *mdi, BonoboWindow *win);
 static void gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context, 
 		                                  gint x, gint y, 
 						  GtkSelectionData *selection_data, 
-				                  guint info, guint time);
+						  guint info, guint time);
+static gboolean gedit_mdi_drag_motion_handler    (GtkWidget        *widget,
+						  GdkDragContext   *context,
+						  gint              x,
+						  gint              y,
+						  guint             time);
+static gboolean gedit_mdi_drag_drop_handler      (GtkWidget        *widget,
+						  GdkDragContext   *context,
+						  gint              x,
+						  gint              y,
+						  guint             time);
 static void gedit_mdi_set_app_toolbar_style 	(BonoboWindow *win);
 static void gedit_mdi_set_app_statusbar_style 	(BonoboWindow *win);
 
@@ -228,12 +237,34 @@ menu_position_under_widget (GtkMenu *menu, int *x, int *y,
 	*y = CLAMP (*y, 0, MAX (0, screen_height - requisition.height));
 }
 
+static void
+tooltip_func (GtkTooltips   *tooltips,
+	      GtkWidget     *menu,
+	      EggRecentItem *item,
+	      gpointer       user_data)
+{
+	gchar *tip;
+	gchar *uri_for_display;
+
+	uri_for_display = egg_recent_item_get_uri_for_display (item);
+	g_return_if_fail (uri_for_display != NULL);
+
+	/* Translators: %s is a URI */
+	tip = g_strdup_printf (_("Open '%s'"), uri_for_display);
+
+	g_free (uri_for_display);
+
+	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (menu), tip, NULL);
+
+	g_free (tip);
+}
+
 static gboolean
 open_button_pressed_cb (GtkWidget *widget,
-			      GdkEventButton *event,
-			      gpointer *user_data)
+			GdkEventButton *event,
+			gpointer *user_data)
 {
-	GtkWidget *menu;
+	static GtkWidget *menu;
 	GeditMDI *mdi;
 
 	g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
@@ -243,7 +274,25 @@ open_button_pressed_cb (GtkWidget *widget,
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 
-	menu = g_object_get_data (G_OBJECT (widget), "recent-menu");
+	if (menu == NULL)
+	{
+		EggRecentViewGtk *view;
+		EggRecentModel *model;
+
+		model = gedit_recent_get_model ();
+
+		menu = gtk_menu_new ();
+		gtk_widget_show (menu);
+		view = egg_recent_view_gtk_new (menu, NULL);
+		g_signal_connect (view, "activate",
+				  G_CALLBACK (gedit_file_open_recent), NULL);
+		egg_recent_view_gtk_show_icons (view, TRUE);
+		egg_recent_view_gtk_show_numbers (view, FALSE);
+		egg_recent_view_gtk_set_tooltip_func (view, tooltip_func, NULL);
+
+		egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
+	}
+
 	gnome_popup_menu_do_popup_modal (menu,
 				menu_position_under_widget, widget,
 				event, widget, widget);
@@ -268,35 +317,10 @@ open_button_key_pressed_cb (GtkWidget *widget,
 	return FALSE;
 }
 
-static void
-tooltip_func (GtkTooltips   *tooltips,
-	      GtkWidget     *menu,
-	      EggRecentItem *item,
-	      gpointer       user_data)
-{
-	gchar *tip;
-	gchar *uri_for_display;
-
-	uri_for_display = egg_recent_item_get_uri_for_display (item);
-	g_return_if_fail (uri_for_display != NULL);
-	
-	/* Translators: %s is a URI */
-	tip = g_strdup_printf (_("Open '%s'"), uri_for_display);
-
-	g_free (uri_for_display);
-
-	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (menu), tip, NULL);
-
-	g_free (tip);
-}
-
 static GtkWidget *
 gedit_mdi_add_open_button (GeditMDI *mdi, BonoboUIComponent *ui_component,
 			 const gchar *path, const gchar *tooltip)
 {
-	GtkWidget *menu;
-	EggRecentViewGtk *view;
-	EggRecentModel *model;
 	GtkWidget *button;
 	
 	static GtkTooltips *button_tooltip = NULL;
@@ -314,20 +338,6 @@ gedit_mdi_add_open_button (GeditMDI *mdi, BonoboUIComponent *ui_component,
 
 	gtk_widget_show_all (GTK_WIDGET (button));
 
-	model = gedit_recent_get_model ();
-
-	menu = gtk_menu_new ();
-	gtk_widget_show (menu);
-	view = egg_recent_view_gtk_new (menu, NULL);
-	g_signal_connect (view, "activate",
-			  G_CALLBACK (gedit_file_open_recent), NULL);
-	egg_recent_view_gtk_show_icons (view, TRUE);
-	egg_recent_view_gtk_show_numbers (view, FALSE);
-	egg_recent_view_gtk_set_tooltip_func (view, tooltip_func, NULL);
-	
-	egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
-	g_object_set_data (G_OBJECT (button), "recent-menu", menu);
-	
 	g_signal_connect_object (button, "key_press_event",
 				 G_CALLBACK (open_button_key_pressed_cb),
 				 mdi, 0);
@@ -410,7 +420,6 @@ gedit_mdi_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
 /**
  * gedit_mdi_new:
  * 
@@ -727,19 +736,40 @@ save_prefs:
 }
 
 static void 
-gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context, 
-		                      gint x, gint y, GtkSelectionData *selection_data, 
-				      guint info, guint time)
+gedit_mdi_drag_data_received_handler (GtkWidget *widget, 
+				      GdkDragContext *context, 
+		                      gint x, 
+				      gint y, 
+				      GtkSelectionData *selection_data, 
+				      guint info, 
+				      guint time)
 {
 	GList *list = NULL;
 	GSList *file_list = NULL;
 	GList *p = NULL;
-	
+	GtkWidget *target_window;
+
 	gedit_debug (DEBUG_MDI, "");
 
 	if (info != TARGET_URI_LIST)
 		return;
-			
+
+	g_return_if_fail (widget != NULL);
+
+	target_window = gtk_widget_get_toplevel (widget);
+	g_return_if_fail (BONOBO_IS_WINDOW (target_window));
+
+	/* Make sure to activate the target window so to load the
+	 * dropped files in the right window */
+	if (target_window != GTK_WIDGET(gedit_get_active_window ()))
+	{
+		GtkWidget *view;
+		view = bonobo_mdi_get_view_from_window (BONOBO_MDI (gedit_mdi),
+							BONOBO_WINDOW (target_window));
+		bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi),
+					    view);
+	}
+					
 	list = gnome_vfs_uri_list_parse (selection_data->data);
 	p = list;
 
@@ -759,12 +789,88 @@ gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context
 	if (file_list == NULL)
 		return;
 
-	gedit_file_open_uri_list (file_list, 0, FALSE);	
+	gedit_file_open_uri_list (file_list, NULL, 0, FALSE);	
 	
 	g_slist_foreach (file_list, (GFunc)g_free, NULL);	
 	g_slist_free (file_list);
 }
 
+/*
+ * Override the gtk_text_view_drag_motion and drag_drop 
+ * functions to get URLs
+ *
+ * If the mime type is text/uri-list, then we will accept
+ * the potential drop, or request the data (depending on the
+ * function).
+ *
+ * If the drag context has any other mime type, then pass the
+ * information onto the GtkTextView's standard handlers.
+ * (widget_class->function_name).
+ *
+ * Fixes bug #89881
+ */
+
+static gboolean 
+gedit_mdi_drag_motion_handler (GtkWidget        *widget,
+			       GdkDragContext   *context,
+			       gint              x,
+			       gint              y,
+			       guint             time)
+{
+	GtkTargetList  *tl;
+	GtkWidgetClass *widget_class;
+	gboolean        result;
+
+	tl = gtk_target_list_new (drag_types, n_drag_types);
+
+	/* If this is a URL, deal with it here, or pass to the text view */
+	if (gtk_drag_dest_find_target (widget, context, tl) != GDK_NONE) 
+	{
+		gdk_drag_status (context, context->suggested_action, time);
+		result = TRUE;
+	} else
+	{
+		widget_class = GTK_WIDGET_GET_CLASS (widget);
+		result = (*widget_class->drag_motion) (widget, context, x, y, time);
+	}
+
+	gtk_target_list_unref (tl);
+
+	return result;
+}
+
+static gboolean 
+gedit_mdi_drag_drop_handler (GtkWidget        *widget,
+			     GdkDragContext   *context,
+			     gint              x,
+			     gint              y,
+			     guint             time)
+{
+	GtkTargetList  *tl;
+	GtkWidgetClass *widget_class;
+	gboolean        result;
+	GdkAtom         target;
+
+	tl = gtk_target_list_new (drag_types, n_drag_types);
+
+	/* If this is a URL, just get the drag data */
+	target = gtk_drag_dest_find_target (widget, context, tl);
+	if (target != GDK_NONE)
+	{
+		gtk_drag_get_data (widget, context, target, time);
+		result = TRUE;
+	}
+	else
+	{
+		widget_class = GTK_WIDGET_GET_CLASS (widget);
+		result = (*widget_class->drag_drop) (widget, context, x, y, time);
+	}
+
+	gtk_target_list_unref (tl);
+
+	return result;
+}
+
 static void
 gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 {
@@ -943,7 +1049,7 @@ gedit_mdi_add_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 	g_signal_connect (G_OBJECT (child), "find_state_changed",
 			  G_CALLBACK (gedit_mdi_child_find_state_changed_handler), 
 			  NULL);
-
+		    
 	return TRUE;
 }
 
@@ -969,15 +1075,26 @@ gedit_mdi_add_view_handler (BonoboMDI *mdi, GtkWidget *view)
 	g_signal_connect (G_OBJECT (text_view), "drag_data_received",
 			  G_CALLBACK (gedit_mdi_drag_data_received_handler), 
 			  NULL);
+	/*
+	 * Get signals before the standard text view functions to deal 
+	 * with urls for text files.
+	 */
+
+	g_signal_connect (G_OBJECT (text_view), "drag_motion",
+			  G_CALLBACK (gedit_mdi_drag_motion_handler), 
+			  NULL);
+
+	g_signal_connect (G_OBJECT (text_view), "drag_drop",
+			  G_CALLBACK (gedit_mdi_drag_drop_handler), 
+			  NULL);
 
 	return TRUE;
 }
 
-
 static gint 
 gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 {
-	GeditDocument* doc;
+	GeditDocument *doc;
 	gboolean close = TRUE;
 	
 	gedit_debug (DEBUG_MDI, "");
@@ -1077,34 +1194,21 @@ gedit_mdi_can_remove_views (GList *views, BonoboWindow *window)
 	{
 		GeditView *view;
 		GeditDocument *doc;
-		gchar *raw_uri;
-		gboolean deleted;
-		
+
 		view = GEDIT_VIEW (l->data);
 		
 		doc = gedit_view_get_document (view);
 		g_return_val_if_fail (doc != NULL, FALSE);
-		
-		deleted = FALSE;
-		
-		raw_uri = gedit_document_get_raw_uri (doc); 
-		if (raw_uri != NULL)
-		{
-			if (gedit_document_is_readonly (doc))
-				deleted = FALSE;
-			else
-				deleted = !gedit_utils_uri_exists (raw_uri);
-		}
-		g_free (raw_uri);
 
-		if (gedit_document_get_modified (doc) || deleted)
+		if (gedit_document_get_modified (doc) ||
+		    gedit_document_get_deleted (doc))
 		{
 			unsaved_docs = g_slist_prepend (unsaved_docs, doc);
 		}
 
 		l = g_list_next (l);
 	}
-	
+
 	if (unsaved_docs == NULL)
 	{
 		l = views;
@@ -1123,7 +1227,7 @@ gedit_mdi_can_remove_views (GList *views, BonoboWindow *window)
 						     
 			l = g_list_next (l);
 		}
-		
+
 		return TRUE;
 	}
 
@@ -1257,16 +1361,42 @@ gedit_mdi_remove_all (GeditMDI *mdi)
 	return ret;
 }
 	
-#define MAX_URI_IN_TITLE_LENGTH 75
+#define MAX_TITLE_LENGTH 100
+
+static gchar *
+get_dirname (const gchar *uri)
+{
+	gchar *res;
+	gchar *str;
+
+	str = g_path_get_dirname (uri);
+	g_return_val_if_fail (str != NULL, ".");
+
+	if ((strlen (str) == 1) && (*str == '.'))
+	{
+		g_free (str);
+		
+		return NULL;
+	}
+
+	res = gedit_utils_replace_home_dir_with_tilde (str);
+
+	g_free (str);
+	
+	return res;
+}
 
 void 
 gedit_mdi_set_active_window_title (BonoboMDI *mdi)
 {
 	BonoboMDIChild *active_child = NULL;
 	GeditDocument *doc = NULL;
-	gchar *docname = NULL;
+	gchar *short_name;
+	gchar *name;
+	gchar *dirname = NULL;
 	gchar *title = NULL;
-	gchar *uri;
+	gint len;
+
 	GtkWidget *active_window;
 	
 	gedit_debug (DEBUG_MDI, "");
@@ -1277,50 +1407,88 @@ gedit_mdi_set_active_window_title (BonoboMDI *mdi)
 
 	doc = GEDIT_MDI_CHILD (active_child)->document;
 	g_return_if_fail (doc != NULL);
-	
-	/* Set active window title */
-	uri = gedit_document_get_uri (doc);
-	g_return_if_fail (uri != NULL);
 
-	/* Truncate the URI so it doesn't get insanely wide. */
-	docname = eel_str_middle_truncate (uri, MAX_URI_IN_TITLE_LENGTH);
-	g_free (uri);
+	short_name = gedit_document_get_short_name (doc);
+	g_return_if_fail (short_name != NULL);
+
+	len = g_utf8_strlen (short_name, -1);
+
+	/* if the name is awfully long, truncate it and be done with it,
+	 * otherwise also show the directory (ellipsized if needed)
+	 */
+	if (len > MAX_TITLE_LENGTH)
+	{
+		name = gedit_utils_str_middle_truncate (short_name, MAX_TITLE_LENGTH);
+		g_free (short_name);
+
+		dirname = NULL;
+	}
+	else
+	{
+		gchar *uri;
+		gchar *str;
+
+		name = short_name;
+
+		uri = gedit_document_get_uri (doc);
+		g_return_if_fail (uri != NULL);
+
+		str = get_dirname (uri);
+		g_free (uri);
+
+		if (str != NULL)
+		{
+			/* use the remaining space for the dir, but use a min of 20 chars
+			 * so that we do not end up with a dirname like "(a...b)".
+			 * This means that in the worst case when the filename is long 99
+			 * we have a title long 99 + 20, but I think it's a rare enough
+			 * case to be acceptable. It's justa darn title afterall :)
+			 */
+			dirname = gedit_utils_str_middle_truncate (str, MAX (20, MAX_TITLE_LENGTH - len));
+			g_free (str);
+		}
+		else
+		{
+			dirname = NULL;
+		}
+	}
 
 	if (gedit_document_get_modified (doc))
 	{
-		title = g_strdup_printf ("%s %s - gedit", docname, _("(modified)"));
+		if (dirname != NULL)
+			title = g_strdup_printf ("*%s (%s) - gedit", name, dirname);
+		else
+			title = g_strdup_printf ("*%s - gedit", name);
 	} 
 	else 
 	{
 		if (gedit_document_is_readonly (doc)) 
 		{
-			title = g_strdup_printf ("%s %s - gedit", docname, _("(readonly)"));
+			if (dirname != NULL)
+				title = g_strdup_printf ("%s [%s] (%s) - gedit", name, _("Read Only"), dirname);
+			else
+				title = g_strdup_printf ("%s [%s] - gedit", name, _("Read Only"));
 		} 
 		else 
 		{
-			title = g_strdup_printf ("%s - gedit", docname);
+			if (dirname != NULL)
+				title = g_strdup_printf ("%s (%s) - gedit", name, dirname);
+			else
+				title = g_strdup_printf ("%s - gedit", name);
 		}
-
 	}
 
 	active_window = GTK_WIDGET (gedit_get_active_window ());
 
-	if (GTK_WIDGET_REALIZED (active_window))
-	{	
-		gchar *short_name;
-		
-		short_name = gedit_document_get_short_name (doc);
-		
-		gdk_window_set_icon_name (active_window->window, short_name);
-		g_free (short_name);
-	}
-
 	gtk_window_set_title (GTK_WINDOW (active_window), title);
-	
-	g_free (docname);
+
+	g_free (dirname);
+	g_free (name);
 	g_free (title);
 }
 
+#undef MAX_TITLE_LENGTH
+
 static 
 void gedit_mdi_child_changed_handler (BonoboMDI *mdi, BonoboMDIChild *old_child)
 {
@@ -1420,9 +1588,6 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	gedit_menus_set_verb_list_sensitive (ui_component, 
 				gedit_menus_all_sensible_verbs, TRUE);
 
-	bonobo_ui_component_set_prop (
-			ui_component, "/menu/View/HighlightMode", "sensitive", "1", NULL);
-	
 	if (active_child == NULL)
 	{
 		gedit_menus_set_verb_list_sensitive (ui_component, 
@@ -1434,9 +1599,13 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 		goto end;
 	}
 
-	gedit_menus_set_verb_sensitive (ui_component, "/commands/DocumentsMoveToNewWindow",
-				(bonobo_mdi_n_children_for_window (active_window) > 1) ? TRUE : FALSE);
-
+	if (gedit_prefs_manager_get_enable_syntax_highlighting ())
+		bonobo_ui_component_set_prop (ui_component, "/menu/View/HighlightMode",
+					      "sensitive", "1", NULL);
+	else
+		bonobo_ui_component_set_prop (ui_component, "/menu/View/HighlightMode",
+					      "sensitive", "0", NULL);
+	
 	doc = GEDIT_MDI_CHILD (active_child)->document;
 	g_return_if_fail (doc != NULL);
 	
@@ -1472,7 +1641,6 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	}
 
 end:
-
 	switch (gedit_mdi_get_state (GEDIT_MDI (mdi)))
 	{
 		case GEDIT_STATE_LOADING:
@@ -1483,6 +1651,8 @@ end:
 			
 			bonobo_ui_component_set_prop (ui_component, 
 					"/menu/View/HighlightMode", "sensitive", "0", NULL);
+		
+			/* FIXME: disable the Recent Files menu - Paolo (Nov 10, 2004) */
 
 			break;
 
@@ -1493,16 +1663,20 @@ end:
 			bonobo_ui_component_set_prop (ui_component, 
 					"/menu/View/HighlightMode", "sensitive", "0", NULL);
 
+			/* FIXME: disable the Recent Files menu - Paolo (Nov 10, 2004) */
+
 			break;
 		default:
 			/* Do nothing */
 			break;
 	}
 
+	gedit_menus_set_verb_sensitive (ui_component, "/commands/DocumentsMoveToNewWindow",
+				(bonobo_mdi_n_views_for_window (active_window) > 1) ? TRUE : FALSE);
+
 	bonobo_ui_component_thaw (ui_component, NULL);
 }
 
-
 static void 
 gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi)
 {
@@ -1636,7 +1810,6 @@ gedit_mdi_hide_output_window_cb (GtkWidget *widget, gpointer user_data)
 				    prefs->output_window_visible);
 }
 
-
 GtkWidget *
 gedit_mdi_get_output_window_from_window (BonoboWindow *win)
 {
@@ -1667,8 +1840,6 @@ gedit_mdi_get_output_window_from_window (BonoboWindow *win)
 	return (r != NULL) ? GTK_WIDGET (r) : NULL;
 }
 
-
-
 static GeditWindowPrefs *
 gedit_window_prefs_new (void)
 {
@@ -1736,7 +1907,6 @@ gedit_window_prefs_save (GeditWindowPrefs *prefs)
 	if ((prefs->statusbar_visible != gedit_prefs_manager_get_statusbar_visible ()) &&
 	    gedit_prefs_manager_statusbar_visible_can_set ())
 		gedit_prefs_manager_set_statusbar_visible (prefs->statusbar_visible);
-
 }
 
 static gchar* 
@@ -1751,7 +1921,7 @@ escape_underscores (const gchar* text)
 
     	length = strlen (text);
 
-	str = g_string_new ("");
+	str = g_string_sized_new (length);
 
   	p = text;
   	end = text + length;
@@ -1789,7 +1959,7 @@ replace_spaces_with_underscores (const gchar* text)
 
     	length = strlen (text);
 
-	str = g_string_new ("");
+	str = g_string_sized_new (length);
 
   	p = text;
   	end = text + length;
@@ -1883,7 +2053,6 @@ language_toggled_handler (
 	gedit_document_set_language (doc, lang);
 }
 
-
 static gchar *
 get_verb_name_for_language (GtkSourceLanguage *lang)
 {
@@ -1909,7 +2078,6 @@ get_verb_name_for_language (GtkSourceLanguage *lang)
 	return verb_name;
 }
 
-
 static void
 add_languages_menu (BonoboMDI *mdi, BonoboWindow *win)
 {
@@ -2061,33 +2229,33 @@ void gedit_mdi_update_languages_menu (BonoboMDI *mdi)
 static void
 update_ui_according_to_state (GeditMDI *mdi)
 {
+	GList *l;
 	GList *views;
 	GdkCursor *cursor;
 	GList *windows;
 
 	gedit_debug (DEBUG_MDI, "");
-	
+
 	/* Upate menus and toolbars */
 	gedit_mdi_set_active_window_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
 
 	/* Update views editability */
 	views = bonobo_mdi_get_views (BONOBO_MDI (mdi));
-	while (views != NULL)
+
+	l = views;
+	while (l != NULL)
 	{
 		GeditView *view;
 		GeditDocument *doc;
 
-		view = GEDIT_VIEW (views->data);
+		view = GEDIT_VIEW (l->data);
 		doc = gedit_view_get_document (view);
-		
+
 		switch (gedit_mdi_get_state (mdi))
 		{
 			case GEDIT_STATE_NORMAL:
 				if (!gedit_document_is_readonly (doc))
-				{
 					gedit_view_set_editable (view, TRUE);
-				}
-
 				break;
 
 			case GEDIT_STATE_LOADING:
@@ -2098,7 +2266,7 @@ update_ui_according_to_state (GeditMDI *mdi)
 				break;
 		}
 
-		views = g_list_next (views);
+		l = g_list_next (l);
 	}
 
 	g_list_free (views);
