@@ -38,15 +38,22 @@
 #include <atk/atk.h>
 #include <gedit/gedit-encodings.h>
 
-void	gedit_utils_flash     (gchar *msg);
+
+/* some common error strings, %s must be a file path */
+#define MISSING_FILE    N_("Could not find \"%s\". Please, reinstall gedit.\n")
+#define MISSING_WIDGETS N_("Could not find the required widgets inside\"%s\". Please, reinstall gedit.\n")
+
+
+void	gedit_utils_flash     (const gchar *msg);
 void	gedit_utils_flash_va  (gchar *format, ...);
 
+void	gedit_utils_set_status    (const gchar *msg);
+void	gedit_utils_set_status_va (gchar *format, ...);
+
+
 gboolean gedit_utils_is_uri_read_only (const gchar* uri);
 gboolean gedit_utils_uri_has_file_scheme (const gchar *uri);
 
-GtkWidget *gedit_button_new_with_stock_image (const gchar* text, 
-					      const gchar* stock_id);
-
 GtkWidget *gedit_dialog_add_button (GtkDialog *dialog, 
 				    const gchar* text, 
 				    const gchar* stock_id, 
@@ -55,20 +62,6 @@ GtkWidget *gedit_dialog_add_button (GtkDialog *dialog,
 gchar *gedit_utils_str_middle_truncate (const gchar *string, 
 					guint truncate_length);
 
-void gedit_utils_error_reporting_loading_file (const gchar *uri, 
-					       const GeditEncoding *encoding,
-					       GError *error,
-					       GtkWindow *parent);
-void gedit_utils_error_reporting_saving_file  (const gchar *uri, 
-					       GError *error,
-					       GtkWindow *parent);
-void gedit_utils_error_reporting_reverting_file (const gchar *uri, 
-					       GError *error,
-					       GtkWindow *parent);
-void gedit_utils_error_reporting_creating_file (const gchar *uri,
-						gint error_code,
-						GtkWindow *parent);
-
 gboolean g_utf8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2);
 
 void gedit_utils_set_atk_name_description (GtkWidget *widget, const gchar *name,
@@ -80,11 +73,20 @@ gboolean gedit_utils_uri_exists (const gchar* text_uri);
 
 gchar *gedit_utils_convert_search_text (const gchar *text);
 
-gboolean gedit_utils_create_empty_file (const gchar *uri);
-
 gchar *gedit_utils_get_stdin (void);
 
-void gedit_warning (gchar *str, GtkWindow *parent);
+void gedit_warning (GtkWindow *parent, gchar *format, ...);
+
+gchar *gedit_utils_str_middle_truncate (const char *string,
+					guint truncate_length);
+
+gchar *gedit_utils_make_valid_utf8 (const char *name);
+
+gchar *gedit_utils_uri_get_basename (const char *uri);
+
+gchar *gedit_utils_uri_get_dirname (const char *uri);
+
+gchar *gedit_utils_replace_home_dir_with_tilde (const gchar *uri);
 
 #endif /* __GEDIT_UTILS_H__ */
 
